// convex-partition-cin.cpp
// Modified version of a CGAL demo (examples/Partition_2/optimal_convex_partition_2.cpp)
// MCS 481 project 2 description version 1.0
#include <CGAL/basic.h>
#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
#include <CGAL/Partition_traits_2.h>
#include <CGAL/partition_2.h>
#include <list>
using namespace std;

typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
typedef CGAL::Partition_traits_2<K>                         Traits;
typedef Traits::Point_2                                     Point;
typedef Traits::Polygon_2                                   Polygon;
typedef list<Polygon>                                       PolygonList;
typedef Polygon::Vertex_iterator                            VertexIterator;
typedef list<Polygon>::const_iterator                       PolygonIterator;

int main( )
{
  Polygon    pgn;
  PolygonList convex_pieces;

  // Read vertex list from stdin and construct polygon
  Point p;
  while (cin >> p) {
    // cout << "Read vertex: " << p << endl;
    pgn.push_back(p);
  }
  
  // CGAL requires CCW orientation for monotone partition
  // If the input polygon is CW oriented, we reverse it
  if (!pgn.is_counterclockwise_oriented())
    pgn.reverse_orientation();

  // Compute optimal convex partition and store results in convex_pieces
  CGAL::optimal_convex_partition_2(pgn.vertices_begin(),
				   pgn.vertices_end(),
				   back_inserter(convex_pieces));
  
  // Print number of polygons in the decomposition
  cout << "Number of convex pieces: " << convex_pieces.size() << endl;

  PolygonIterator pi;
  int n=1;
  for (pi = convex_pieces.begin(); pi != convex_pieces.end(); pi++) {
    cout << "Piece " << n++ << ":" << endl;
    VertexIterator vi;
    for (vi = (*pi).vertices_begin(); vi != (*pi).vertices_end(); vi++) {
      cout << "\t" << *vi << endl;
    }
  }
  return 0;
}
