// monotone-partition-cin.cpp
// Modified version of a CGAL demo (examples/Partition_2/y_monotone_partition_2.cpp)
// MCS 481 project 2 description version 1.0
#include <CGAL/basic.h>
#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
#include <CGAL/Partition_traits_2.h>
#include <CGAL/partition_2.h>
#include <list>
using namespace std;

typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
typedef CGAL::Partition_traits_2<K>                         Traits;
typedef Traits::Point_2                                     Point;
typedef Traits::Polygon_2                                   Polygon;
typedef list<Polygon>                                       PolygonList;
typedef Polygon::Vertex_iterator                            VertexIterator;
typedef list<Polygon>::const_iterator                       PolygonIterator;

int main( )
{
  Polygon    pgn;
  PolygonList monotone_pieces;

  // Read vertex list from stdin and construct polygon
  Point p;
  while (cin >> p) {
    // cout << "Read vertex: " << p << endl;
    pgn.push_back(p);
  }
  
  // CGAL requires CCW orientation for monotone partition
  // If the input polygon is CW oriented, we reverse it
  if (!pgn.is_counterclockwise_oriented())
    pgn.reverse_orientation();

  // Compute y-monotone partition and store results in monotone_pieces
  CGAL::y_monotone_partition_2(pgn.vertices_begin(),
			       pgn.vertices_end(),
			       back_inserter(monotone_pieces));
  
  // Print number of polygons in the decomposition
  cout << "Number of monotone pieces: " << monotone_pieces.size() << endl;

  PolygonIterator pi;
  int n=1;
  for (pi = monotone_pieces.begin(); pi != monotone_pieces.end(); pi++) {
    cout << "Piece " << n++ << ":" << endl;
    VertexIterator vi;
    for (vi = (*pi).vertices_begin(); vi != (*pi).vertices_end(); vi++) {
      cout << "\t" << *vi << endl;
    }
  }
  return 0;
}
